Feature: download and verify an update-description file
  As a Tails developer
  I work on the Tails update system
  I want to make sure the update-description downloader and verifier works properly

  Background:
    Given a usable temporary directory
    And a running "Tails", version "0.11", targetted at "s390x", using channel "stable"
    And a HTTP random port
    And a HTTPS random port
    And a trusted Certificate Authority
    And a trusted OpenPGP signing key pair
    And a trusted, but expired OpenPGP signing key pair
    And a trusted OpenPGP signing key pair created in the future
    And an untrusted OpenPGP signing key pair

  Scenario: Failed download of a non-existing update-description
    Given a HTTPS server with a valid SSL certificate
    And no update-description that matches the running Tails
    When I download and check this update-description file
    Then it should fail
    And I should be told "Could not download"

  Scenario: Successful download and verification
    Given a HTTPS server with a valid SSL certificate
    And an update-description that matches the running Tails
    And a valid signature made by a trusted key
    When I download and check this update-description file
    Then it should succeed
    And the update-description content should be printed

  Scenario: Failed download due to untrusted SSL certificate
    Given a HTTPS server with an invalid SSL certificate
    And an update-description that matches the running Tails
    And a valid signature made by a trusted key
    When I download and check this update-description file
    Then it should fail
    And I should be told "certificate verify failed"

  Scenario: Failed download due to expired SSL certificate
    Given a HTTPS server with an expired SSL certificate
    And an update-description that matches the running Tails
    And a valid signature made by a trusted key
    When I download and check this update-description file
    Then it should fail
    And I should be told "certificate verify failed"

  Scenario: Failed download due to SSL certificate that is not valid yet
    Given a HTTPS server with a not-valid-yet SSL certificate
    And an update-description that matches the running Tails
    And a valid signature made by a trusted key
    When I download and check this update-description file
    Then it should fail
    And I should be told "certificate verify failed"

  Scenario: Failed download when server redirects to cleartext HTTP
    Given a HTTPS server with a valid SSL certificate, that redirects to 127.0.0.2 over cleartext HTTP
    And a HTTP server on 127.0.0.2
    And an update-description that matches the running Tails
    And a valid signature made by a trusted key
    When I download and check this update-description file
    Then it should fail
    And I should be told "Access to 'http' URIs has been disabled"

  Scenario: Successful download, failed verification of an invalid signature made by a trusted key
    Given a HTTPS server with a valid SSL certificate
    And an update-description that matches the running Tails
    And an invalid signature made by a trusted key
    When I download and check this update-description file
    Then it should fail
    And I should be told "Invalid signature"

  Scenario: Successful download, failed verification of valid signature made by an untrusted key
    Given a HTTPS server with a valid SSL certificate
    And an update-description that matches the running Tails
    And a valid signature made by an untrusted key
    When I download and check this update-description file
    Then it should fail
    And I should be told "Invalid signature"

  Scenario: Successful download of the update-description, failed download of the signature
    Given a HTTPS server with a valid SSL certificate
    And an update-description that matches the running Tails
    When I download and check this update-description file
    Then it should fail
    And I should be told "Could not download"

  Scenario: Failed download of the update-description from a non-existing server
    Given a non-existing web server
    When I download and check an update-description file from this server
    Then it should fail
    And I should be told "Could not download"

  Scenario: Failed download of an update-description that is bigger than expected
    Given a HTTPS server with a valid SSL certificate
    And an update-description that is too big
    And a valid signature made by a trusted key
    When I download and check this update-description file
    Then it should fail
    And the downloaded content should be not be much bigger than 8192
    And I should be told "but the downloaded content .* should be smaller than .*"

  Scenario: Failed download of a signature that is bigger than expected
    Given a HTTPS server with a valid SSL certificate
    And an update-description that matches the running Tails
    And a signature that is too big
    When I download and check this update-description file
    Then it should fail
    And the downloaded content should be not be much bigger than 8192
    And I should be told "but the downloaded content .* should be smaller than .*"

  Scenario: Successful download, signature made in the future
    Given a HTTPS server with a valid SSL certificate
    And an update-description that matches the running Tails
    And a valid signature made in the future by a trusted key
    When I download and check this update-description file
    Then it should succeed
    And the update-description content should be printed

  Scenario: Successful download, signature made by an expired key
    Given a HTTPS server with a valid SSL certificate
    And an update-description that matches the running Tails
    And a valid signature made by a trusted, but expired key
    When I download and check this update-description file
    Then it should succeed
    And the update-description content should be printed

  Scenario: Successful download, signature made by a key created in the future
    Given a HTTPS server with a valid SSL certificate
    And an update-description that matches the running Tails
    And a valid signature made by a trusted key created in the future
    When I download and check this update-description file
    Then it should succeed
    And the update-description content should be printed

  Scenario: Well-signed update-description does not match the running Tails' product name
    Given a HTTPS server with a valid SSL certificate
    And an update-description that has a different "product_name" than the running Tails
    And a valid signature made by a trusted key
    When I download and check this update-description file
    Then it should fail
    And I should be told "Does not match running system"

  Scenario: Well-signed update-description does not match the running Tails' version
    Given a HTTPS server with a valid SSL certificate
    And an update-description that has a different "product_version" than the running Tails
    And a valid signature made by a trusted key
    When I download and check this update-description file
    Then it should fail
    And I should be told "Does not match running system"

  Scenario: Well-signed update-description does not match the running Tails' architecture
    Given a HTTPS server with a valid SSL certificate
    And an update-description that has a different "build_target" than the running Tails
    And a valid signature made by a trusted key
    When I download and check this update-description file
    Then it should fail
    And I should be told "Does not match running system"

  Scenario: Well-signed update-description does not match the running Tails' channel
    Given a HTTPS server with a valid SSL certificate
    And an update-description that has a different "channel" than the running Tails
    And a valid signature made by a trusted key
    When I download and check this update-description file
    Then it should fail
    And I should be told "Does not match running system"
